@@ -40,20 +40,21 @@
 
 /*
 **
-** mm.h -- memory map library API header
+**  mm.h -- Shared Memory library API header
 **
 */
 
 #ifndef MM_H 
 #define MM_H 1
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
 **  ____ Public Part (I) of the API ________________________
 */
 
-#define MM_VERSION_STR "0.9.0"
-#define MM_VERSION_NUM 0x000900
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -67,6 +68,9 @@
 #if !defined(NULL)
 #define NULL (void *)0
 #endif
+#if !defined(NUL)
+#define NUL '\0'
+#endif
 
 /*
 **  ____ Private Part of the API ___________________________
@@ -83,36 +87,51 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#if defined(MM_OS_SunOS)
+#ifdef MM_OS_SUNOS
 #include <memory.h>
 /* SunOS lacks prototypes */
-int getpagesize(void);
-int munmap(caddr_t addr, int len);
-int ftruncate(int fd, off_t length);
-int flock(int fd, int operation);
+extern int getpagesize(void);
+extern int munmap(caddr_t addr, int len);
+extern int ftruncate(int fd, off_t length);
+extern int flock(int fd, int operation);
+extern char *strerror (int err);
 #endif
 
 #if !defined(offsetof)
 #define offsetof(type,member) ((size_t)(&((type *)0)->member))
 #endif
 
-#define FAIL           goto cus
-#define BEGIN_FAILURE  cus:
+#if !defined(HAVE_MEMCPY) 
+#if defined(HAVE_BCOPY)
+#define memcpy(to,from,len) bcopy(from,to,len)
+#else
+#define memcpy(to,from,len) \
+        { int i; for (i = 0; i < (len); i++) *((to)+i) = *((from)+i); }
+#endif
+#endif
+#if !defined(HAVE_MEMSET)
+#define memset(to,ch,len) \
+        { int i; for (i = 0; i < (len); i++) *((to)+i) = (ch); }
+#endif
+
+#define ERR(type,str)   mm_lib_error_set(type,str)
+#define FAIL(type,str)  { ERR(type,str); goto cus; }
+#define BEGIN_FAILURE   cus:
 #define END_FAILURE   
 
-#if defined(PATH_MAX)
+#if defined(HAVE_PATH_MAX)
 #define MM_MAXPATH PATH_MAX
-#elif defined(_POSIX_PATH_MAX)
+#elif defined(HAVE__POSIX_PATH_MAX)
 #define MM_MAXPATH _POSIX_PATH_MAX
-#elif defined(MAXPATHLEN)
+#elif defined(HAVE_MAXPATHLEN)
 #define MM_MAXPATH MAXPATHLEN
 #else
 #define MM_MAXPATH 2048
 #endif
 
-#if defined(CHILD_MAX)
+#if defined(HAVE_CHILD_MAX)
 #define MM_MAXCHILD CHILD_MAX
-#elif defined(_POSIX_CHILD_MAX)
+#elif defined(HAVE__POSIX_CHILD_MAX)
 #define MM_MAXCHILD _POSIX_CHILD_MAX
 #else
 #define MM_MAXCHILD 512
@@ -124,6 +143,9 @@ int flock(int fd, int operation);
 #if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
 #define MAP_ANON MAP_ANONYMOUS
 #endif
+#if !defined(MAP_FAILED)
+#define MAP_FAILED ((void *)-1)
+#endif
 #endif
 
 #if defined(MM_SHMT_IPCSHM) || defined(MM_SEMT_IPCSEM)
@@ -136,18 +158,28 @@ int flock(int fd, int operation);
 
 #if defined(MM_SEMT_IPCSEM)
 #include <sys/sem.h>
+#ifndef HAVE_UNION_SEMUN
+union semun {
+    int val;
+    struct semid_ds *buf;
+    u_short *array;
+};
+#endif
 #endif
 
 #ifdef MM_SEMT_FLOCK
 #include <sys/file.h>
 #endif
 
-#define MM_CORE_FILEMODE    (S_IRUSR|S_IWUSR)
-#define MM_CORE_MINSIZE     (1*1024)             /* 1 KB */
-#define MM_CORE_MAXSIZE     (16*1024*1024)       /* 16 MB */
-#define MM_DEFAULT_PAGESIZE 8192                 /* 8 KB */
-#define MM_DEFAULT_SIZE     (1*1024*1024)        /* 1MB */
-#define MM_DEFAULT_FILE     "/tmp/mm.core.%d"    /* %d is PID */
+#define MM_ALLOC_MINSIZE         (1024*8) 
+#define MM_ALLOC_MAXSIZE         (MM_SHM_MAXSEGSIZE-SIZEOF_mem_pool)
+#define MM_CORE_FILEMODE         (S_IRUSR|S_IWUSR)
+#define MM_CORE_DEFAULT_PAGESIZE (1024*8)
+#define MM_CORE_DEFAULT_FILE     "/tmp/mm.core.%d"  /* %d is PID */
+
+#define MM_ERR_ALLOC    1
+#define MM_ERR_CORE     2
+#define MM_ERR_SYSTEM   4
 
 /* 
  * Define a union with types which are likely to have the longest
@@ -167,13 +199,13 @@ typedef union mem_word mem_word;
  * Define the structure used for memory chunks
  */
 union mem_chunk_mc_u {
-	struct mem_chunk *mc_next; /* really used when it's free         */
-	mem_word          mc_base; /* virtually used when it's allocated */
+    struct mem_chunk *mc_next; /* really used when it's free         */
+    mem_word          mc_base; /* virtually used when it's allocated */
 };
 struct mem_chunk {
     size_t mc_size;      /* physical size   */
     size_t mc_usize;     /* user known size */
-	union mem_chunk_mc_u mc_u;
+    union mem_chunk_mc_u mc_u;
 };
 typedef struct mem_chunk mem_chunk;
 #define SIZEOF_mem_chunk (sizeof(mem_chunk)-sizeof(union mem_chunk_mc_u))
@@ -209,23 +241,28 @@ typedef struct mem_core_fd mem_core_fd;
  * required)
  */
 struct mem_core {
-   size_t      mc_size;
-   size_t      mc_usize;
-   pid_t       mc_pid;
-   int         mc_fdmem;
+   size_t       mc_size;
+   size_t       mc_usize;
+   pid_t        mc_pid;
+   int          mc_fdmem;
 #if defined(MM_SHMT_MMFILE)
-   char        mc_fnmem[MM_MAXPATH];
+   char         mc_fnmem[MM_MAXPATH];
 #endif
 #if !defined(MM_SEMT_FLOCK)
-   int         mc_fdsem;
+   int          mc_fdsem;
 #endif
 #if defined(MM_SEMT_FLOCK)
-   mem_core_fd mc_fdsem[MM_MAXCHILD];
+   mem_core_fd  mc_fdsem[MM_MAXCHILD];
+#endif
+#if defined(MM_SEMT_IPCSEM)
+   int          mc_fdsem_rd;
+   int          mc_readers;
+   mm_lock_mode mc_lockmode;
 #endif
 #if defined(MM_SEMT_FLOCK) || defined(MM_SEMT_FCNTL)
-   char        mc_fnsem[MM_MAXPATH];
+   char         mc_fnsem[MM_MAXPATH];
 #endif
-   mem_word    mc_base;
+   mem_word     mc_base;
 };
 typedef struct mem_core mem_core;
 #define SIZEOF_mem_core (sizeof(mem_core)-SIZEOF_mem_word)
@@ -242,10 +279,14 @@ typedef mem_pool MM;
 typedef void MM;
 #endif
 
+typedef enum { 
+    MM_LOCK_RD, MM_LOCK_RW
+} mm_lock_mode;
+
 /* Global Malloc-Replacement API */
-int     MM_create(void);
+int     MM_create(size_t size, char *file);
 void    MM_destroy(void);
-int     MM_lock(void);
+int     MM_lock(mm_lock_mode mode);
 int     MM_unlock(void);
 void   *MM_malloc(size_t size);
 void   *MM_realloc(void *ptr, size_t size);
@@ -253,11 +294,14 @@ void    MM_free(void *ptr);
 void   *MM_calloc(size_t number, size_t size);
 char   *MM_strdup(const char *str);
 size_t  MM_sizeof(void *ptr);
+size_t  MM_maxsize(void);
+size_t  MM_available(void);
+char   *MM_error(void);
 
 /* Standard Malloc-Style API */
 MM     *mm_create(size_t size, char *file);
 void    mm_destroy(MM *mm);
-int     mm_lock(MM *mm);
+int     mm_lock(MM *mm, mm_lock_mode mode);
 int     mm_unlock(MM *mm);
 void   *mm_malloc(MM *mm, size_t size);
 void   *mm_realloc(MM *mm, void *ptr, size_t size);
@@ -265,17 +309,28 @@ void    mm_free(MM *mm, void *ptr);
 void   *mm_calloc(MM *mm, size_t number, size_t size);
 char   *mm_strdup(MM *mm, const char *str);
 size_t  mm_sizeof(MM *mm, void *ptr);
-#ifdef MM_DEBUG
+size_t  mm_maxsize(void);
+size_t  mm_available(MM *mm);
+char   *mm_error(void);
 void    mm_display_info(MM *mm);
-#endif
 
 /* Low-Level Shared Memory API */
-size_t  mm_core_align2page(size_t size);
-size_t  mm_core_align2word(size_t size);
 void   *mm_core_create(size_t size, char *file);
 void    mm_core_delete(void *core);
 size_t  mm_core_size(void *core);
-int     mm_core_lock(void *core);
+int     mm_core_lock(void *core, mm_lock_mode mode);
 int     mm_core_unlock(void *core);
+size_t  mm_core_maxsegsize(void);
+size_t  mm_core_align2page(size_t size);
+size_t  mm_core_align2word(size_t size);
+
+/* Internal Library API */
+void    mm_lib_error_set(unsigned int, const char *str);
+char   *mm_lib_error_get(void);
+int     mm_lib_version(void);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* MM_H */
