@@ -40,21 +40,13 @@
 
 /*
 **
-** mm_alloc.c -- High-Level Memory Allocation
+**  mm_alloc.c -- Standard Malloc-Style API
 **
 */
 
 #define MM_PRIVATE
 #include "mm.h"
 
-#if !defined(MM_NA)
-
-/*
-**
-**  Standard Malloc-Style API
-**
-*/
-
 /*
  * Create a memory pool
  */
@@ -68,9 +60,11 @@ MM *mm_create(size_t usize, char *file)
     if (usize < 0)
         return NULL;
     if (usize == 0)
-        usize = MM_DEFAULT_SIZE;
-    if (usize <= MM_CORE_MINSIZE)
-        usize = MM_CORE_MINSIZE;
+        usize = MM_ALLOC_MAXSIZE;
+    if (usize <= MM_ALLOC_MINSIZE)
+        usize = MM_ALLOC_MINSIZE;
+    if (usize >= MM_ALLOC_MAXSIZE)
+        usize = MM_ALLOC_MAXSIZE;
 
     /* determine size */
     size = mm_core_align2page(usize+SIZEOF_mem_pool);
@@ -106,24 +100,23 @@ void mm_destroy(MM *mm)
     return;
 }
 
-int mm_lock(MM *mm)
+int mm_lock(MM *mm, mm_lock_mode mode)
 {
     if (mm == NULL);
-        return NULL;
-    return mm_core_lock((void *)mm);
+        return FALSE;
+    return mm_core_lock((void *)mm, mode);
 }
 
 int mm_unlock(MM *mm)
 {
     if (mm == NULL);
-        return NULL;
+        return FALSE;
     return mm_core_unlock((void *)mm);
 }
 
 /*
  * Display debugging information
  */
-#if defined(MM_DEBUG)
 void mm_display_info(MM *mm)
 {
     mem_chunk *mc;
@@ -131,7 +124,7 @@ void mm_display_info(MM *mm)
     int nAlloc;
     int i;
 
-    mm_core_lock((void *)mm);
+    mm_core_lock((void *)mm, MM_LOCK_RD);
     mc = &(mm->mp_freechunks);
     nFree   = 0;
     while (mc->mc_u.mc_next != NULL) {
@@ -141,7 +134,7 @@ void mm_display_info(MM *mm)
     nAlloc = mm->mp_offset-SIZEOF_mem_pool-nFree;
 
     fprintf(stderr, "Information for MM\n");
-    fprintf(stderr, "    memory area     = 0x%x - 0x%x\n", (unsigned int)mm, (unsigned int)mm+mm->mp_size);
+    fprintf(stderr, "    memory area     = 0x%lx - 0x%lx\n", (unsigned long)mm, (unsigned long)(mm+mm->mp_size));
     fprintf(stderr, "    memory size     = %d\n", mm->mp_size);
     fprintf(stderr, "    memory offset   = %d\n", mm->mp_offset);
     fprintf(stderr, "    bytes spare     = %d\n", mm->mp_size-mm->mp_offset);
@@ -153,19 +146,19 @@ void mm_display_info(MM *mm)
     fprintf(stderr, "    List of free chunks:\n");
     if (mm->mp_freechunks.mc_usize > 0) {
         mc = &(mm->mp_freechunks);
-        i = 0;
+        i = 1;
         while (mc->mc_u.mc_next != NULL) {
             mc = mc->mc_u.mc_next;
-            fprintf(stderr, "        chunk #%03d: 0x%x-0x%x (%d bytes)\n", 
-                    i, (unsigned int)mc, (unsigned int)mc+mc->mc_size, mc->mc_size);
+            fprintf(stderr, "        chunk #%03d: 0x%lx-0x%lx (%d bytes)\n", 
+                    i++, (unsigned long)mc, (unsigned long)(mc+mc->mc_size), mc->mc_size);
         }
     }
-    else
-        fprintf(stderr, "        <empty-list>\n"), 
+    else {
+        fprintf(stderr, "        <empty-list>\n");
+    }
     mm_core_unlock((void *)mm);
     return;
 }
-#endif
 
 /*
  * Insert a chunk to the list of free chunks. Algorithm used is:
@@ -179,7 +172,7 @@ static void mm_insert_chunk(MM *mm, mem_chunk *mcInsert)
     mem_chunk *mcPrev;
     mem_chunk *mcNext;
     
-    mm_core_lock((void *)mm);
+    mm_core_lock((void *)mm, MM_LOCK_RW);
     mc = &(mm->mp_freechunks);
     while (mc->mc_u.mc_next != NULL && (char *)(mc->mc_u.mc_next) < (char *)mcInsert)
         mc = mc->mc_u.mc_next;
@@ -229,7 +222,7 @@ static mem_chunk *mm_retrieve_chunk(MM *mm, size_t size)
 
     if (mm->mp_freechunks.mc_usize == 0)
         return NULL;
-    mm_core_lock((void *)mm);
+    mm_core_lock((void *)mm, MM_LOCK_RW);
     mc = &(mm->mp_freechunks);
     mcMinLast = mc;
     mcMin     = NULL;
@@ -281,6 +274,7 @@ void *mm_malloc(MM *mm, size_t usize)
         return mc;
     }
     if ((mm->mp_size - mm->mp_offset) < size) {
+        ERR(MM_ERR_ALLOC, "Out of memory");
         errno = ENOMEM;
         return NULL; 
     }
@@ -382,87 +376,38 @@ size_t mm_sizeof(MM *mm, void *ptr)
 }
 
 /*
-**
-**  Global Malloc-Replacement API
-**
-*/
-
-static MM *mm_global = NULL;
-
-int MM_create(void)
+ * Determine maximum size of an allocateable memory poo
+ */
+size_t mm_maxsize(void)
 {
-    if (mm_global != NULL)
-        return FALSE;
-    if ((mm_global = mm_create(0, NULL)) == NULL)
-        return FALSE;
-    return TRUE;
+    return MM_ALLOC_MAXSIZE;
 }
 
-void MM_destroy(void)
+/*
+ * Determine available memory
+ */
+size_t mm_available(MM *mm)
 {
-    if (mm_global == NULL)
-        return;
-    mm_destroy(mm_global);
-    mm_global = NULL;
-    return;
+    mem_chunk *mc;
+    int nFree;
+
+    mm_core_lock((void *)mm, MM_LOCK_RD);
+    nFree = mm->mp_size-mm->mp_offset;
+    mc = &(mm->mp_freechunks);
+    while (mc->mc_u.mc_next != NULL) {
+        mc = mc->mc_u.mc_next;
+        nFree += mc->mc_size;
+    }
+    mm_core_unlock((void *)mm);
+    return nFree;
 }
 
-int MM_lock(void)
+/*
+ * Return last error string
+ */
+char *mm_error(void)
 {
-    if (mm_global == NULL)
-        return NULL;
-    return mm_lock(mm_global);
+    return mm_lib_error_get();
 }
 
-int MM_unlock(void)
-{
-    if (mm_global == NULL)
-        return NULL;
-    return mm_unlock(mm_global);
-}
-
-void *MM_malloc(size_t size)
-{
-    if (mm_global == NULL)
-        return NULL;
-    return mm_malloc(mm_global, size);
-}
-
-void *MM_realloc(void *ptr, size_t size)
-{
-    if (mm_global == NULL)
-        return NULL;
-    return mm_realloc(mm_global, ptr, size);
-}
-
-void MM_free(void *ptr)
-{
-    if (mm_global == NULL)
-        return;
-    mm_free(mm_global, ptr);
-    return;
-}
-
-void *MM_calloc(size_t number, size_t size)
-{
-    if (mm_global == NULL)
-        return NULL;
-    return mm_calloc(mm_global, number, size);
-}
-
-char *MM_strdup(const char *str)
-{
-    if (mm_global == NULL)
-        return NULL;
-    return mm_strdup(mm_global, str);
-}
-
-size_t MM_sizeof(void *ptr)
-{
-    if (mm_global == NULL)
-        return -1;
-    return mm_sizeof(mm_global, ptr);
-}
-
-#endif /* MM_NA */
 /*EOF*/
